/****************************************************************************
**
** Copyright (C) 1991-2008 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include <GL/gl.h>
#include "sysinfo.h"


// Defines
#ifdef WIN32
	#define OS "OS"
	#define CPU "PROCESSOR_ARCHITECTURE"	// x86
#else
	#define OS "OSTYPE"
	#define CPU "CPU"						// i686
	extern char **environ;
#endif


// Globals
extern OGLWidget *oglWid;		// OpenGL-Viewer
extern float MHz;				// CPU clock was calculated in main
extern int numCPU;				// Number of CPU's/cores calculated in main


static QString cpuStr;
static bool oldwin=true;
static bool oglhw;


// Localization stuff
// --------------------------------------------------------------------------
static QString yes;
static QString no;


// Needed, because a "pure" QObject is not translated!
void initSysInfo() {
	yes=QObject::tr("Yes");
	no=QObject::tr("No");
}


// Maximum texture size available
static int maxTex() {
	int max;
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max);
	return max;
}


// Check system out
// --------------------------------------------------------------------------
QString systemInfo() {
	QString tmp, numStr;

	// Check OS and CPU
	tmp="OS = ";
	#ifdef WIN32
		if(getenv(OS)!=NULL) {
			tmp+=getenv(OS)+(QString)("\n");
			cpuStr=(QString)getenv(CPU);
			tmp+="CPU = " + cpuStr + (QString)("\n");
		}
		else {
			tmp+="Windows < XP\n";
			tmp+="CPU = ?\n";
		}
	#else
		tmp+=(QString)getenv(OS)+(QString)("\n");
		cpuStr=(QString)getenv(CPU);
		tmp+="CPU = " + cpuStr + (QString)("\n");
		oldwin=false;
	#endif

	// Number of CPU's/cores was calculated in main at program-start
	tmp+="Cores = ";
	tmp+=numStr.number(numCPU)+"\n";

	// MHz was checked by main() && aniedi->getclock() at program-start
	#ifndef WIN32
	tmp+="MHz = ";
	if(MHz>0.0f && MHz<10000.0f)
		tmp+=numStr.number(MHz, 'f', 2)+"\n";
	else
		tmp+="?\n";
	#endif

	if(getenv("MACHTYPE")!=NULL)
		tmp+="MACHTYPE = "+(QString)getenv("MACHTYPE")+(QString)("\n");
	else
		tmp+="MACHTYPE = ?\n";

	tmp+="Hardware OpenGL : ";
	if(oglWid->format().directRendering()) {
		tmp+=yes+"\n\n";
		oglhw=true;
	}
	else {
		tmp+=no+"\n\n";
		oglhw=false;
	}

	// Check miscellaneous OpenGL properties
	tmp+="Double buffering : ";
	if(oglWid->format().doubleBuffer()) tmp+=yes+"\n"; else tmp+=no+"\n";

	tmp+="RGBA mode : ";
	if(oglWid->format().rgba()) tmp+=yes+"\n"; else tmp+=no+"\n";

	tmp+="Overlay support : ";
	if(oglWid->format().hasOpenGLOverlays()) tmp+=yes+"\n"; else tmp+=no+"\n";

	tmp+="Multisample support : ";
	if(oglWid->format().sampleBuffers()) tmp+=yes+"\n"; else tmp+=no+"\n";

	tmp+="Depth buffer : ";
	if(oglWid->format().depth()) tmp+=yes+"\n"; else tmp+=no+"\n";
	tmp+="Depth buffer size = ";
	if(oglWid->format().depth())
		tmp+=numStr.number(oglWid->format().depthBufferSize())+" KB\n";
	else
		tmp+="?\n";

	tmp+="Stencil buffer : ";
	if(oglWid->format().stencil()) tmp+=yes+"\n"; else tmp+=no+"\n";
	tmp+="Stencil buffer size = ";
	if(oglWid->format().stencil())
		tmp+=numStr.number(oglWid->format().stencilBufferSize())+" KB\n";
	else
		tmp+="?\n";

	tmp+="Alpha buffer : ";
	if(oglWid->format().alpha()) tmp+=yes+"\n"; else tmp+=no+"\n";
	tmp+="Alpha buffer size = ";
	if(oglWid->format().alpha())
		tmp+=numStr.number(oglWid->format().alphaBufferSize())+" KB\n";
	else
		tmp+="?\n";

	tmp+="Accumulation buffer : ";
	if(oglWid->format().accum()) tmp+=yes+"\n"; else tmp+=no+"\n";
	tmp+="Accumulation buffer size = ";
	if(oglWid->format().accum())
		tmp+=numStr.number(oglWid->format().accumBufferSize())+" KB\n";
	else
		tmp+="?\n";

	tmp+="Max. texture size = ";
	tmp+=numStr.number(maxTex())+"\n";

	return tmp;
}


// Check CPU
bool cpux86() {
	systemInfo();

	if(	cpuStr.startsWith("x86") ||		// Windows XP
		cpuStr.startsWith("i586") ||
		cpuStr.startsWith("i686") )		// Linux
		return true;

	return false;
}


// Check Windows XP
bool winXP() {
	systemInfo();
	return (!oldwin);
}


// Check hardware accelerated OpenGL
bool oglhw_avail() {
	systemInfo();
	return oglhw;
}
