/****************************************************************************
**
** Copyright (C) 1991-2008 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _TICKS_H
#define _TICKS_H


// #############################################################
// CPU-ticks implemented for: x86, x86_64, PowerPC, Mips
// #############################################################

class TICKS {

public:
	TICKS() { cor=0ULL; start(); stop(); cor=t; start(); }
	~TICKS() {}

	void start() { t=fetch(); t=fetch(); t=fetch(); t=fetch(); t=fetch(); }
	unsigned long long stop() { t=fetch()-t; if(t>cor) return t-=cor; return t=0ULL; }
	unsigned long long ticks() { return t; }

private:
	unsigned long long t, cor;
	unsigned long long fetch();
};


// ########################################################################
// Hardware dependent CPU-cycle-counters
// ########################################################################

#if defined(__i386__) || defined(__x86_64__)
inline unsigned long long TICKS::fetch() {
	#ifdef __INTEL_COMPILER
	return _rdtsc();
	#else
	unsigned long high, low;
	asm volatile(
	"rdtsc \n\t"
	: "=a" (low),
	"=d" (high));
	return ((unsigned long long)high << 32) | (low);
	#endif
}

#elif (defined(__mips__) && (defined(MIPSEL) || defined (__MIPSEL__)))
inline unsigned long long TICKS::fetch() {
	unsigned long long tmp;
	__asm__ __volatile__("mfc0 %0,$9; nop" : "=r" (tmp));
	return tmp;
}

#elif defined(__powerpc__)
inline unsigned long long TICKS::fetch(void) {
	unsigned long long result=0;
	unsigned long upper, lower, tmp;
	__asm__ volatile(
		"loop:                  \n"
		"\tmftbu   %0           \n"
		"\tmftb    %1           \n"
		"\tmftbu   %2           \n"
		"\tcmpw    %2,%0        \n"
		"\tbne     loop         \n"
		: "=r"(upper),"=r"(lower),"=r"(tmp)
	);
	result = upper;
	result = result<<32;
	result = result|lower;
	return result;
}
#endif


// _TICKS_H
#endif
